@@ -92,7 +92,11 @@ static pthread_key_t	res_state_key;
 static void
 SPF_dns_resolv_thread_term(void *arg)
 {
+#if HAVE_DECL_RES_NDESTROY
+	res_ndestroy( (struct __res_state *)arg );
+#else
 	res_nclose( (struct __res_state *)arg );
+#endif
 	free(arg);
 }
 
@@ -182,8 +186,8 @@ SPF_dns_resolv_debug(SPF_dns_server_t *spf_dns_server, ns_rr rr,
 			}
 			/* XXX I think this is wrong/unsafe. Shevek. */
 			/* XXX doesn't parse the different TXT "sections" */
-			SPF_debugf("TXT: (%d) \"%.*s\"",
-					rdlen, rdlen - 1, rdata + 1);
+			SPF_debugf("TXT: (%lu) \"%.*s\"",
+					(unsigned long)rdlen, (int)rdlen - 1, rdata + 1);
 			break;
 
 		case ns_t_ptr:
@@ -207,7 +211,8 @@ SPF_dns_resolv_debug(SPF_dns_server_t *spf_dns_server, ns_rr rr,
 
 /**
  * Can return NULL on out-of-memory condition.
- * Should return an NXDOMAIN or appropriate rr in all other error cases.
+ * Should return a HOST_NOT_FOUND or appropriate rr in all other
+ * error cases.
  */
 static SPF_dns_rr_t *
 SPF_dns_resolv_lookup(SPF_dns_server_t *spf_dns_server,
@@ -247,9 +252,11 @@ SPF_dns_resolv_lookup(SPF_dns_server_t *spf_dns_server,
 	if (res_spec == NULL) {
 		res_state = (struct __res_state *)
 						malloc(sizeof(struct __res_state));
+		/* XXX The interface doesn't allow to communicate back failure
+		 * to allocate memory, but SPF_errorf aborts anyway. */
 		if (! res_state)
-			SPF_errorf("Failed to allocate %d bytes for res_state",
-							sizeof(struct __res_state));
+			SPF_errorf("Failed to allocate %lu bytes for res_state",
+							(unsigned long)sizeof(struct __res_state));
 		memset(res_state, 0, sizeof(struct __res_state));
 		if (res_ninit(res_state) != 0)
 			SPF_error("Failed to call res_ninit()");
@@ -311,7 +318,7 @@ SPF_dns_resolv_lookup(SPF_dns_server_t *spf_dns_server,
 							domain, rr_type, 0, SPF_h_errno);
 		}
 		else if (dns_len > responselen) {
-			u_char	*tmp;
+			void	*tmp;
 			/* We managed a lookup but our buffer was too small. */
 			responselen = dns_len + (dns_len >> 1);
 #if 0
@@ -406,9 +413,9 @@ SPF_dns_resolv_lookup(SPF_dns_server_t *spf_dns_server,
 
 			rdlen = ns_rr_rdlen(rr);
 			if (spf_dns_server->debug > 1)
-				SPF_debugf("name: %s  type: %d  class: %d  ttl: %d  rdlen: %d",
+				SPF_debugf("name: %s  type: %d  class: %d  ttl: %d  rdlen: %lu",
 						ns_rr_name(rr), ns_rr_type(rr), ns_rr_class(rr),
-						ns_rr_ttl(rr), rdlen);
+						ns_rr_ttl(rr), (unsigned long)rdlen);
 
 			if (rdlen <= 0)
 				continue;
@@ -615,8 +622,8 @@ SPF_dns_resolv_new(SPF_dns_server_t *layer_below,
 #if HAVE_DECL_RES_NINIT
 	pthread_once(&res_state_control, SPF_dns_resolv_init_key);
 #else
-	if ( res_init() != 0 ) {
-		perror("res_init");
+	if (res_init() != 0) {
+		SPF_warning("Failed to call res_init()");
 		return NULL;
 	}
 #endif
