@@ -40,17 +40,54 @@
 #include "spf_dns_internal.h"
 #include "spf_server.h"
 
-
 static SPF_errcode_t
-SPF_i_set_smtp_comment(SPF_response_t *spf_response)
+SPF_i_set_explanation(SPF_response_t *spf_response)
 {
 	SPF_server_t	*spf_server;
 	SPF_request_t	*spf_request;
 	SPF_record_t	*spf_record;
 	SPF_errcode_t	 err;
 	char			*buf;
-	int				 buflen;
-	int				 len;
+	size_t			 buflen;
+
+	SPF_ASSERT_NOTNULL(spf_response);
+	spf_request = spf_response->spf_request;
+	SPF_ASSERT_NOTNULL(spf_request);
+	spf_server = spf_request->spf_server;
+	SPF_ASSERT_NOTNULL(spf_server);
+
+	spf_record = spf_response->spf_record_exp;
+	SPF_ASSERT_NOTNULL(spf_record);
+
+	if (spf_response->explanation)
+		free(spf_response->explanation);
+	spf_response->explanation = NULL;
+
+	buflen = SPF_SMTP_COMMENT_SIZE + 1;
+	buf = malloc(buflen);
+	if (buf == NULL)
+		return SPF_E_NO_MEMORY;
+	memset(buf, '\0', buflen);
+
+	err = SPF_request_get_exp(spf_server, spf_request,
+					spf_response, spf_record, &buf, &buflen);
+	if (err != SPF_E_SUCCESS) {
+		free(buf);
+		return err;
+	}
+
+	spf_response->explanation = buf;
+
+	return SPF_E_SUCCESS;
+}
+
+static SPF_errcode_t
+SPF_i_set_smtp_comment(SPF_response_t *spf_response)
+{
+	SPF_server_t	*spf_server;
+	SPF_request_t	*spf_request;
+	SPF_errcode_t	 err;
+	char			 buf[SPF_SMTP_COMMENT_SIZE];
 
 	SPF_ASSERT_NOTNULL(spf_response);
 	spf_request = spf_response->spf_request;
@@ -66,44 +103,34 @@ SPF_i_set_smtp_comment(SPF_response_t *spf_response)
 	switch (spf_response->result) {
 		case SPF_RESULT_FAIL:
 		case SPF_RESULT_SOFTFAIL:
+		case SPF_RESULT_NEUTRAL:
 		case SPF_RESULT_NONE:
-			spf_record = spf_response->spf_record_exp;
-			SPF_ASSERT_NOTNULL(spf_record);
 
-			buflen = SPF_SMTP_COMMENT_SIZE + 1;
-			buf = malloc(buflen);
-			memset(buf, '\0', buflen);
+			err = SPF_i_set_explanation(spf_response);
+			if (err != SPF_E_SUCCESS)
+				return err;
 
-			err = SPF_request_get_exp(spf_server, spf_request,
-							spf_response, spf_record, &buf, &buflen);
+			memset(buf, '\0', sizeof(buf));
+			snprintf(buf, SPF_SMTP_COMMENT_SIZE, "%s : Reason: %s",
+					spf_response->explanation,
+					SPF_strreason(spf_response->reason));
+			buf[SPF_SMTP_COMMENT_SIZE - 1] = '\0';
 
-			/* buf should now be malloc'd */
-			if (buf == NULL || buf[0] == '\0')
-				break;
-			/* Someone might have realloc'd us smaller? */
-			if (buflen < SPF_SMTP_COMMENT_SIZE + 1)
-				buf = realloc(buf, SPF_SMTP_COMMENT_SIZE + 1);
+			/* It doesn't really hurt much if this fails. */
+			spf_response->smtp_comment = strdup(buf);
+			if (! spf_response->smtp_comment)
+				return SPF_E_NO_MEMORY;
 
-			len = strlen(buf);
-			if (len < SPF_SMTP_COMMENT_SIZE)
-				snprintf(&buf[len], SPF_SMTP_COMMENT_SIZE - len - 1,
-						" : Reason: %s",
-						SPF_strreason(spf_response->reason));
-			buf[SPF_SMTP_COMMENT_SIZE] = '\0';
-
-			spf_response->smtp_comment = SPF_sanitize(spf_server, buf);
 			break;
 		case SPF_RESULT_INVALID:
-		case SPF_RESULT_NEUTRAL:
 		case SPF_RESULT_PASS:
 		case SPF_RESULT_TEMPERROR:
 		case SPF_RESULT_PERMERROR:
 		default:
-			err = SPF_E_SUCCESS;
 			break;
 	}
 
-	return err;
+	return SPF_E_SUCCESS;
 }
 
 static SPF_errcode_t
@@ -209,19 +236,22 @@ SPF_i_set_header_comment(SPF_response_t *spf_response)
 		break;
 
 	case SPF_RESULT_PERMERROR:
-		snprintf( p, p_end - p, "error in processing during lookup of %s: %s",
-					  spf_source, SPF_strerror( spf_response->err ) );
+		snprintf(p, p_end - p, "error in processing during lookup of %s: %s",
+					  spf_source, SPF_strerror(spf_response->err));
 		break;
 
 	case SPF_RESULT_NEUTRAL:
+		snprintf(p, p_end - p, "%s is neither permitted nor denied by %s",
+				ip, spf_source);
+		break;
 	case SPF_RESULT_NONE:
-		snprintf( p, p_end - p, "%s is neither permitted nor denied by %s",
-				  ip, spf_source );
+		snprintf(p, p_end - p, "%s does not provide an SPF record",
+				spf_source);
 		break;
 
 	case SPF_RESULT_TEMPERROR:
-		snprintf( p, p_end - p, "encountered temporary error during SPF processing of %s",
-				  spf_source );
+		snprintf(p, p_end - p, "encountered temporary error during SPF processing of %s",
+				spf_source );
 		break;
 
 
@@ -231,8 +261,8 @@ SPF_i_set_header_comment(SPF_response_t *spf_response)
 		break;
 	}
 
-	if( spf_source )
-		free( spf_source );
+	if (spf_source)
+		free(spf_source);
 
 	spf_response->header_comment = SPF_sanitize(spf_server, buf);
 
@@ -316,7 +346,7 @@ SPF_i_set_received_spf(SPF_response_t *spf_response)
 		
 
 		/* FIXME: Add in full compiler errors. */
-	#if 0
+#if 0
 		/* add in the optional compiler error keyword */
 		if ( output.err_msg != NULL ) {
 			p += snprintf( p, p_end - p, " problem=%s;", output.err_msg );
@@ -326,7 +356,7 @@ SPF_i_set_received_spf(SPF_response_t *spf_response)
 			p += snprintf( p, p_end - p, " problem=%s;", c_results.err_msg );
 			if ( p_end - p <= 0 ) break;
 		}
-	#endif
+#endif
 
 		/* FIXME  should the explanation string be included in the header? */
 
@@ -346,6 +376,14 @@ SPF_i_set_received_spf(SPF_response_t *spf_response)
 #define DONE_PERMERR(err) DONE(SPF_RESULT_PERMERROR,SPF_REASON_NONE,err)
 #define DONE_MECH(result) DONE(result, SPF_REASON_MECH, SPF_E_SUCCESS)
 
+/**
+ * This must be called with EITHER
+ * spf_response->spf_record_exp != NULL
+ *   OR
+ * result in { SPF_RESULT_PASS SPF_RESULT_INVALID
+ *		SPF_RESULT_TEMPERROR SPF_RESULT_PERMERROR }
+ * or the library will abort when it tries to generate an explanation.
+ */
 SPF_errcode_t
 SPF_i_done(SPF_response_t *spf_response,
 	SPF_result_t result, SPF_reason_t reason, SPF_errcode_t err)
@@ -554,7 +592,7 @@ SPF_record_interpret(SPF_record_t *spf_record,
 	int				 m;			/* Mechanism iterator */
 	SPF_mech_t		*mech;
 	SPF_data_t		*data;
-	SPF_data_t		*data_end;
+	SPF_data_t		*data_end;	/* XXX Replace with size_t data_len */
 
 	/* Where to insert the local policy (whitelist) */
 	SPF_mech_t		*local_policy;	/* Not the local policy */
@@ -586,6 +624,7 @@ SPF_record_interpret(SPF_record_t *spf_record,
 
 	int				max_ptr;
 	int				max_mx;
+	int				max_exceeded;
 
 	char			 ip4_buf[ INET_ADDRSTRLEN ];
 	char			 ip6_buf[ INET6_ADDRSTRLEN ];
@@ -600,6 +639,8 @@ SPF_record_interpret(SPF_record_t *spf_record,
 	spf_server = spf_record->spf_server;
 	SPF_ASSERT_NOTNULL(spf_server);
 
+	SPF_ASSERT_NOTNULL(spf_response->spf_record_exp);
+
 	if (depth > 20)
 		return DONE_PERMERR(SPF_E_RECURSIVE);
 
@@ -705,50 +746,50 @@ SPF_record_interpret(SPF_record_t *spf_record,
 		}												\
 	} while(0)
 #define SPF_FREE_LOOKUP_DATA() \
-	do { if (buf != NULL) free(buf);} while(0)
+	do { if (buf != NULL) { free(buf); buf = NULL; } } while(0)
 
 
 	resolver = spf_server->resolver;
 
 	mech = spf_record->mech_first;
-	for( m = 0; m < spf_record->num_mech; m++ ) {
+	for (m = 0; m < spf_record->num_mech; m++) {
 
 		/* This is as good a place as any. */
-		if ( spf_response->num_dns_mech > spf_server->max_dns_mech
-			 || spf_response->num_dns_mech > SPF_MAX_DNS_MECH ) {
+		/* XXX Rip this out and put it into a macro which can go into inner loops. */
+		if (spf_response->num_dns_mech > spf_server->max_dns_mech) {
 			SPF_FREE_LOOKUP_DATA();
-			return DONE( SPF_RESULT_TEMPERROR, SPF_REASON_NONE, SPF_E_BIG_DNS );
+			return DONE(SPF_RESULT_PERMERROR, SPF_REASON_NONE, SPF_E_BIG_DNS);
 		}
 
-		data = SPF_mech_data( mech );
-		data_end = SPF_mech_end_data( mech );
+		data = SPF_mech_data(mech);
+		data_end = SPF_mech_end_data(mech);
 
-		switch( mech->mech_type )
-		{
+		switch (mech->mech_type) {
 		case MECH_A:
 			SPF_ADD_DNS_MECH();
 			SPF_MAYBE_SKIP_CIDR();
 			SPF_GET_LOOKUP_DATA();
 
-			if ( spf_request->client_ver == AF_INET )
+			if (spf_request->client_ver == AF_INET)
 				fetch_ns_type = ns_t_a;
 			else
 				fetch_ns_type = ns_t_aaaa;
 
-			rr_a = SPF_dns_lookup(resolver, lookup, fetch_ns_type, TRUE );
+			rr_a = SPF_dns_lookup(resolver, lookup, fetch_ns_type, TRUE);
 
-			if ( spf_server->debug )
-				SPF_debugf( "found %d A records for %s  (herrno: %d)",
-						rr_a->num_rr, lookup, rr_a->herrno );
+			if (spf_server->debug)
+				SPF_debugf("found %d A records for %s  (herrno: %d)",
+						rr_a->num_rr, lookup, rr_a->herrno);
 
-			if( rr_a->herrno == TRY_AGAIN ) {
+			if (rr_a->herrno == TRY_AGAIN) {
 				SPF_dns_rr_free(rr_a);
 				SPF_FREE_LOOKUP_DATA();
 				return DONE_TEMPERR(SPF_E_DNS_ERROR); /* REASON_MECH */
 			}
 
 			for (i = 0; i < rr_a->num_rr; i++) {
-				if ( rr_a->rr_type != fetch_ns_type )
+				/* XXX Should this be hoisted? */
+				if (rr_a->rr_type != fetch_ns_type)
 					continue;
 
 				if (spf_request->client_ver == AF_INET) {
@@ -775,29 +816,32 @@ SPF_record_interpret(SPF_record_t *spf_record,
 			SPF_MAYBE_SKIP_CIDR();
 			SPF_GET_LOOKUP_DATA();
 
-			rr_mx = SPF_dns_lookup(resolver, lookup, ns_t_mx, TRUE );
+			rr_mx = SPF_dns_lookup(resolver, lookup, ns_t_mx, TRUE);
 
-			if ( spf_server->debug )
-				SPF_debugf( "found %d MX records for %s  (herrno: %d)",
-						rr_mx->num_rr, lookup, rr_mx->herrno );
+			if (spf_server->debug)
+				SPF_debugf("found %d MX records for %s  (herrno: %d)",
+						rr_mx->num_rr, lookup, rr_mx->herrno);
 
-			if( rr_mx->herrno == TRY_AGAIN ) {
+			if (rr_mx->herrno == TRY_AGAIN) {
 				SPF_dns_rr_free(rr_mx);
 				SPF_FREE_LOOKUP_DATA();
 				return DONE_TEMPERR(SPF_E_DNS_ERROR);
 			}
 
+			/* The maximum number of MX records we will inspect. */
 			max_mx = rr_mx->num_rr;
-			if ( max_mx > spf_server->max_dns_mx )
-				max_mx = spf_server->max_dns_mx;
-			if ( max_mx > SPF_MAX_DNS_MX )
-				max_mx = SPF_MAX_DNS_MX;
+			max_exceeded = 0;
+			if (max_mx > spf_server->max_dns_mx) {
+				max_exceeded = 1;
+				max_mx = SPF_server_get_max_dns_mx(spf_server);
+			}
 
-			for( j = 0; j < max_mx; j++ ) {
-				if ( rr_mx->rr_type != ns_t_mx )
+			for (j = 0; j < max_mx; j++) {
+				/* XXX Should this be hoisted? */
+				if (rr_mx->rr_type != ns_t_mx)
 					continue;
 
-				if ( spf_request->client_ver == AF_INET )
+				if (spf_request->client_ver == AF_INET)
 					fetch_ns_type = ns_t_a;
 				else
 					fetch_ns_type = ns_t_aaaa;
@@ -805,40 +849,39 @@ SPF_record_interpret(SPF_record_t *spf_record,
 				rr_a = SPF_dns_lookup(resolver, rr_mx->rr[j]->mx,
 									   fetch_ns_type, TRUE );
 
-				if ( spf_server->debug )
-					SPF_debugf( "%d: found %d A records for %s  (herrno: %d)",
-							j, rr_a->num_rr, rr_mx->rr[j]->mx, rr_a->herrno );
-				if( rr_a->herrno == TRY_AGAIN ) {
+				if (spf_server->debug)
+					SPF_debugf("%d: found %d A records for %s  (herrno: %d)",
+							j, rr_a->num_rr, rr_mx->rr[j]->mx, rr_a->herrno);
+				if (rr_a->herrno == TRY_AGAIN) {
 					SPF_dns_rr_free(rr_mx);
 					SPF_dns_rr_free(rr_a);
 					SPF_FREE_LOOKUP_DATA();
-					return DONE_TEMPERR( SPF_E_DNS_ERROR );
+					return DONE_TEMPERR(SPF_E_DNS_ERROR);
 				}
 
-				for( i = 0; i < rr_a->num_rr; i++ ) {
-					if ( rr_a->rr_type != fetch_ns_type )
+				for (i = 0; i < rr_a->num_rr; i++) {
+					/* XXX Should this be hoisted? */
+					if (rr_a->rr_type != fetch_ns_type)
 						continue;
 
-					if ( spf_request->client_ver == AF_INET )
-					{
-						if ( SPF_i_match_ip4( spf_server, spf_request, mech, rr_a->rr[i]->a ) )
-						{
+					if (spf_request->client_ver == AF_INET) {
+						if (SPF_i_match_ip4(spf_server, spf_request, mech,
+										rr_a->rr[i]->a)) {
 							SPF_dns_rr_free(rr_mx);
 							SPF_dns_rr_free(rr_a);
 							SPF_FREE_LOOKUP_DATA();
-							return DONE( mech->prefix_type, SPF_REASON_MECH,
-										 SPF_E_SUCCESS );
+							return DONE(mech->prefix_type, SPF_REASON_MECH,
+										 SPF_E_SUCCESS);
 						}
 					}
-					else
-					{
-						if ( SPF_i_match_ip6( spf_server, spf_request, mech, rr_a->rr[i]->aaaa ) )
-						{
+					else {
+						if (SPF_i_match_ip6(spf_server, spf_request, mech,
+										rr_a->rr[i]->aaaa)) {
 							SPF_dns_rr_free(rr_mx);
 							SPF_dns_rr_free(rr_a);
 							SPF_FREE_LOOKUP_DATA();
-							return DONE( mech->prefix_type, SPF_REASON_MECH,
-										 SPF_E_SUCCESS );
+							return DONE(mech->prefix_type, SPF_REASON_MECH,
+										 SPF_E_SUCCESS);
 						}
 					}
 				}
@@ -846,56 +889,66 @@ SPF_record_interpret(SPF_record_t *spf_record,
 			}
 
 			SPF_dns_rr_free( rr_mx );
+			if (max_exceeded) {
+				SPF_FREE_LOOKUP_DATA();
+				return DONE(SPF_RESULT_PERMERROR, SPF_REASON_NONE, SPF_E_BIG_DNS);
+			}
 			break;
 
 		case MECH_PTR:
 			SPF_ADD_DNS_MECH();
 			SPF_GET_LOOKUP_DATA();
 
-			if ( spf_request->client_ver == AF_INET ) {
+			if (spf_request->client_ver == AF_INET) {
 				rr_ptr = SPF_dns_rlookup(resolver,
 								spf_request->ipv4, ns_t_ptr, TRUE);
 
-				if ( spf_server->debug ) {
+				if (spf_server->debug) {
 					INET_NTOP(AF_INET, &spf_request->ipv4.s_addr,
 										ip4_buf, sizeof(ip4_buf));
 					SPF_debugf("got %d PTR records for %s (herrno: %d)",
 							rr_ptr->num_rr, ip4_buf, rr_ptr->herrno);
 				}
 
-				if( rr_ptr->herrno == TRY_AGAIN ) {
+				if (rr_ptr->herrno == TRY_AGAIN) {
 					SPF_dns_rr_free(rr_ptr);
 					SPF_FREE_LOOKUP_DATA();
 					return DONE_TEMPERR(SPF_E_DNS_ERROR);
 				}
 
 
+				/* The maximum number of PTR records we will inspect. */
 				max_ptr = rr_ptr->num_rr;
-				if ( max_ptr > spf_server->max_dns_ptr )
-					max_ptr = spf_server->max_dns_ptr;
-				if ( max_ptr > SPF_MAX_DNS_PTR )
-					max_ptr = SPF_MAX_DNS_PTR;
+				max_exceeded = 0;
+				if (max_ptr > spf_server->max_dns_ptr) {
+					max_exceeded = 1;
+					max_ptr = SPF_server_get_max_dns_ptr(spf_server);
+				}
+
+				for (i = 0; i < max_ptr; i++) {
+					/* XXX MX has a 'continue' case here which should be hoisted. */
 
-				for( i = 0; i < max_ptr; i++ ) {
 					rr_a = SPF_dns_lookup(resolver,
-							rr_ptr->rr[i]->ptr, ns_t_a, TRUE );
+							rr_ptr->rr[i]->ptr, ns_t_a, TRUE);
 
-					if ( spf_server->debug )
+					if (spf_server->debug)
 						SPF_debugf( "%d:  found %d A records for %s  (herrno: %d)",
 								i, rr_a->num_rr, rr_ptr->rr[i]->ptr, rr_a->herrno );
-					if( rr_a->herrno == TRY_AGAIN ) {
+					if (rr_a->herrno == TRY_AGAIN) {
 						SPF_dns_rr_free(rr_ptr);
 						SPF_dns_rr_free(rr_a);
 						SPF_FREE_LOOKUP_DATA();
 						return DONE_TEMPERR( SPF_E_DNS_ERROR );
 					}
 
-					for( j = 0; j < rr_a->num_rr; j++ ) {
-						if ( spf_server->debug ) {
-							INET_NTOP( AF_INET, &rr_a->rr[j]->a.s_addr,
+					for (j = 0; j < rr_a->num_rr; j++) {
+						/* XXX MX has a 'continue' case here which should be hoisted. */
+
+						if (spf_server->debug) {
+							INET_NTOP(AF_INET, &rr_a->rr[j]->a.s_addr,
 											ip4_buf, sizeof(ip4_buf));
-							SPF_debugf( "%d: %d:  found %s",
-									i, j, ip4_buf );
+							SPF_debugf("%d: %d:  found %s",
+									i, j, ip4_buf);
 						}
 
 						if (rr_a->rr[j]->a.s_addr ==
@@ -905,13 +958,18 @@ SPF_record_interpret(SPF_record_t *spf_record,
 								SPF_dns_rr_free(rr_ptr);
 								SPF_dns_rr_free(rr_a);
 								SPF_FREE_LOOKUP_DATA();
-								return DONE_MECH( mech->prefix_type );
+								return DONE_MECH(mech->prefix_type);
 							}
 						}
 					}
 					SPF_dns_rr_free(rr_a);
 				}
 				SPF_dns_rr_free(rr_ptr);
+
+				if (max_exceeded) {
+					SPF_FREE_LOOKUP_DATA();
+					return DONE(SPF_RESULT_PERMERROR, SPF_REASON_NONE, SPF_E_BIG_DNS);
+				}
 			}
 
 			else if ( spf_request->client_ver == AF_INET6 ) {
@@ -932,18 +990,21 @@ SPF_record_interpret(SPF_record_t *spf_record,
 
 
 				max_ptr = rr_ptr->num_rr;
-				if ( max_ptr > spf_server->max_dns_ptr )
-					max_ptr = spf_server->max_dns_ptr;
-				if ( max_ptr > SPF_MAX_DNS_PTR )
-					max_ptr = SPF_MAX_DNS_PTR;
+				max_exceeded = 0;
+				if (max_ptr > spf_server->max_dns_ptr) {
+					max_ptr = SPF_server_get_max_dns_ptr(spf_server);
+					max_exceeded = 1;
+				}
+
+				for (i = 0; i < max_ptr; i++) {
+					/* XXX MX has a 'continue' case here which should be hoisted. */
 
-				for( i = 0; i < max_ptr; i++ ) {
 					rr_aaaa = SPF_dns_lookup(resolver,
 							rr_ptr->rr[i]->ptr, ns_t_aaaa, TRUE);
 
 					if ( spf_server->debug )
-						SPF_debugf( "%d:  found %d AAAA records for %s  (herrno: %d)",
-								i, rr_aaaa->num_rr, rr_ptr->rr[i]->ptr, rr_aaaa->herrno );
+						SPF_debugf("%d:  found %d AAAA records for %s  (herrno: %d)",
+								i, rr_aaaa->num_rr, rr_ptr->rr[i]->ptr, rr_aaaa->herrno);
 					if( rr_aaaa->herrno == TRY_AGAIN ) {
 						SPF_dns_rr_free(rr_ptr);
 						SPF_dns_rr_free(rr_aaaa);
@@ -952,6 +1013,7 @@ SPF_record_interpret(SPF_record_t *spf_record,
 					}
 
 					for( j = 0; j < rr_aaaa->num_rr; j++ ) {
+						/* XXX MX has a 'continue' case here which should be hoisted. */
 						if ( spf_server->debug ) {
 							INET_NTOP(AF_INET6, &rr_aaaa->rr[j]->aaaa.s6_addr,
 											ip6_buf, sizeof(ip6_buf));
@@ -974,8 +1036,14 @@ SPF_record_interpret(SPF_record_t *spf_record,
 					SPF_dns_rr_free(rr_aaaa);
 				}
 				SPF_dns_rr_free(rr_ptr);
+
+				if (max_exceeded) {
+					SPF_FREE_LOOKUP_DATA();
+					return DONE(SPF_RESULT_PERMERROR, SPF_REASON_NONE, SPF_E_BIG_DNS);
+				}
 			}
 
+
 			break;
 
 		case MECH_INCLUDE:
@@ -1022,9 +1090,14 @@ SPF_record_interpret(SPF_record_t *spf_record,
 				if (spf_record_subr)
 					SPF_record_free(spf_record_subr);
 				SPF_FREE_LOOKUP_DATA();
-				return DONE_TEMPERR( err );
+				if (err == SPF_E_DNS_ERROR)
+					return DONE_TEMPERR( err );
+				else
+					return DONE_PERMERR( err );
 			}
 
+			SPF_ASSERT_NOTNULL(spf_record_subr);
+
 			/*
 			 * If we are a redirect which is not within the scope
 			 * of any include.
@@ -1033,11 +1106,19 @@ SPF_record_interpret(SPF_record_t *spf_record,
 				save_spf_response = NULL;
 				if (spf_response->spf_record_exp == spf_record)
 					spf_response->spf_record_exp = spf_record_subr;
+				SPF_ASSERT_NOTNULL(spf_response->spf_record_exp);
 			}
 			else {
 				save_spf_response = spf_response;
 				spf_response = SPF_response_new(spf_request);
+				if (! spf_response) {
+					if (spf_record_subr)
+						SPF_record_free(spf_record_subr);
+					SPF_FREE_LOOKUP_DATA();
+					return DONE_TEMPERR(SPF_E_NO_MEMORY);
+				}
 				spf_response->spf_record_exp = spf_record;
+				SPF_ASSERT_NOTNULL(spf_response->spf_record_exp);
 			}
 			/*
 			 * find out whether this configuration passes
@@ -1149,14 +1230,17 @@ SPF_record_interpret(SPF_record_t *spf_record,
 						rr_a->num_rr, lookup, rr_a->herrno );
 
 			if( rr_a->herrno == TRY_AGAIN ) {
+				SPF_dns_rr_free(rr_a);
 				SPF_FREE_LOOKUP_DATA();
 				return DONE_TEMPERR(SPF_E_DNS_ERROR);
 			}
 			if ( rr_a->num_rr > 0 ) {
+				SPF_dns_rr_free(rr_a);
 				SPF_FREE_LOOKUP_DATA();
 				return DONE_MECH(mech->prefix_type);
 			}
 
+			SPF_dns_rr_free(rr_a);
 			break;
 
 		case MECH_ALL:
