@@ -54,12 +54,13 @@ SPF_record_print(SPF_record_t *spf_record)
 		return SPF_E_SUCCESS;
 	}
 
-    SPF_infof( "SPF header:  version: %d  mech %d/%d  mod %d/%d  len=%d",
+    SPF_infof( "SPF header:  version: %d  mech %d/%u  mod %d/%u  len=%u",
 	    spf_record->version,
-	    spf_record->num_mech, spf_record->mech_len, 
-	    spf_record->num_mod, spf_record->mod_len,
-	    sizeof(SPF_record_t) + spf_record->mech_len
-	    + spf_record->mod_len);
+	    (int)spf_record->num_mech, (unsigned int)spf_record->mech_len, 
+	    (int)spf_record->num_mod, (unsigned int)spf_record->mod_len,
+	    (unsigned int)(sizeof(SPF_record_t)
+				+ spf_record->mech_len
+				+ spf_record->mod_len));
 
     err = SPF_record_stringify(spf_record, &prt_buf, &prt_len);
     if ( err == SPF_E_RESULT_UNKNOWN )
@@ -81,7 +82,7 @@ SPF_record_print(SPF_record_t *spf_record)
 void SPF_print_sizeof(void)
 {
     // SPF_infof( "sizeof(SPF_rec_header_t)=%u", sizeof(SPF_rec_header_t));
-    SPF_infof( "sizeof(SPF_mech_t)=%u", sizeof(SPF_mech_t));
-    SPF_infof( "sizeof(SPF_data_t)=%u", sizeof(SPF_data_t));
-    SPF_infof( "sizeof(SPF_mod_t)=%u", sizeof(SPF_mod_t));
+    SPF_infof( "sizeof(SPF_mech_t)=%lu", (unsigned long)sizeof(SPF_mech_t));
+    SPF_infof( "sizeof(SPF_data_t)=%lu", (unsigned long)sizeof(SPF_data_t));
+    SPF_infof( "sizeof(SPF_mod_t)=%lu", (unsigned long)sizeof(SPF_mod_t));
 }
